@@ -588,7 +588,7 @@ impl LuaUserData for Editor {
             // If you can't render the editor, you're pretty much done for anyway
             let Size { w, mut h } = crate::ui::size().unwrap_or(Size { w: 0, h: 0 });
             h = h.saturating_sub(1 + editor.push_down);
-            let _ = editor.terminal.hide_cursor();
+            editor.terminal.hide_cursor();
             // Apply render and restore cursor
             if editor.try_doc().is_some() {
                 let _ = editor.render_feedback_line(w, h);
@@ -596,10 +596,10 @@ impl LuaUserData for Editor {
             if let Some(doc) = editor.try_doc() {
                 let max = editor.dent();
                 if let Some(Loc { x, y }) = doc.cursor_loc_in_screen() {
-                    let _ = editor.terminal.goto(x + max, y + editor.push_down);
+                    editor.terminal.goto(x + max, y + editor.push_down);
                 }
             }
-            let _ = editor.terminal.show_cursor();
+            editor.terminal.show_cursor();
             let _ = editor.terminal.flush();
             Ok(())
         });
@@ -607,14 +607,14 @@ impl LuaUserData for Editor {
             // If you can't render the editor, you're pretty much done for anyway
             let Size { w, mut h } = crate::ui::size().unwrap_or(Size { w: 0, h: 0 });
             h = h.saturating_sub(1 + editor.push_down);
-            let _ = editor.terminal.hide_cursor();
+            editor.terminal.hide_cursor();
             let _ = editor.render_status_line(lua, w, h);
             // Apply render and restore cursor
             let max = editor.dent();
             if let Some(Loc { x, y }) = editor.doc().cursor_loc_in_screen() {
-                let _ = editor.terminal.goto(x + max, y + editor.push_down);
+                editor.terminal.goto(x + max, y + editor.push_down);
             }
-            let _ = editor.terminal.show_cursor();
+            editor.terminal.show_cursor();
             let _ = editor.terminal.flush();
             Ok(())
         });
